#include "chirp_reli.h"
#include "auth_all.h"
#include "debug.h"
#include "chirp_array.h"
#include "xmalloc.h"
#include "nvpair.h"
#include "catalog_query.h"

#include <time.h>
#include <stdio.h>
#include <time.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/wait.h>


struct chirp_dual_array {
	char host[CHIRP_LINE_MAX];
	char path[CHIRP_LINE_MAX];
	int width;
	int height;
	int element_size;
	struct chirp_array *row_oriented;
	struct chirp_array *col_oriented;
};


/* create a new array at the given path, return zero on failure*/
struct chirp_dual_array * chirp_dual_array_create( const char * host, const char *path, int width, int height, int element_size )
{
	struct chirp_dual_array *array;
	array = xxmalloc(sizeof(*array));
	
	FILE *out = fopen("/tmp/d_array.crp","w");
	if(out!=NULL)
		fprintf(out,"%s\n%s\n%d\n%d\n%d\n",host,path,width,height,element_size);
	else
	{
		printf("Error: can not write to /tmp\n");
		return 0;
	}
	
	int result;
	time_t stoptime = time(0)+300;
	
	strcpy(array->host,host);
	strcpy(array->path,path);
	
	array->width = width;
	array->height = height;
	array->element_size = element_size;

	
	
	fclose(out);
	out = fopen("/tmp/d_array.crp","r");
	
	result = chirp_reli_mkdir(host,path,0755,stoptime);
	
	char PATH[CHIRP_LINE_MAX];
	sprintf(PATH, "%s%s%s%s",path, "/",path,".mtdata");

	struct stat64 info;
	fstat64(fileno(out),&info);
	if(chirp_reli_putfile(host,PATH,out,info.st_mode,info.st_size,stoptime)!=info.st_size)
	{
		fprintf(stderr,"couldn't put %s: %s\n",path,strerror(errno));
		return 0;
	}
	
	
	array->row_oriented = xxmalloc(sizeof((struct chirp_array*)array->row_oriented));
	array->col_oriented = xxmalloc(sizeof((struct chirp_array*)array->col_oriented));
	
	char tempPath[CHIRP_LINE_MAX];
	sprintf(tempPath ,"%s_row", path);
	array->row_oriented = chirp_array_create( host, tempPath, width, height, element_size);
	sprintf(tempPath ,"%s_col", path);
	array->col_oriented = chirp_array_create( host, tempPath, height, width, element_size);
	
	return array;
}

/* open an existing array, return zero on failure */
struct chirp_dual_array * chirp_dual_array_open( const char * host, const char *path )
{
	struct chirp_dual_array *array;
	array = xxmalloc(sizeof(*array));
	
	array->row_oriented = xxmalloc(sizeof((struct chirp_array*)array->row_oriented));
	array->col_oriented = xxmalloc(sizeof((struct chirp_array*)array->col_oriented));

	time_t stoptime = time(0)+300;
	if(!chirp_client_connect(host,1,stoptime))
	{
		printf("Error: cannot connect to host %s\n",host);
		return 0;
	}

	FILE *tempfile;

	tempfile = fopen("/tmp/tempfile","w");
	if(tempfile==NULL)
	{
		printf("Cannot write to /tmp\n");
		return 0;
	}
	else
	{
		char PATH[CHIRP_LINE_MAX];
		sprintf(PATH, "%s%s%s%s",path, "/",path,".mtdata");
		int result = chirp_reli_getfile(host,PATH,tempfile,stoptime);
		//cannot get metadata file
		if(result<0)
		{
			printf("%s %s %d %i %s chirp_getfile failed\n",host,host,result,errno,strerror(errno));
			return 0;
		}
		else
		{
			fclose(tempfile);
			tempfile = fopen("/tmp/tempfile","r");
			char tmp[CHIRP_LINE_MAX];
			fscanf(tempfile,"%s",tmp);
			strcpy(array->host,tmp);
			fscanf(tempfile,"%s",tmp);
			strcpy(array->path,tmp);
			fscanf(tempfile,"%d",&array->width);
			fscanf(tempfile,"%d",&array->height);
			fscanf(tempfile,"%d",&array->element_size);
			
			char tempPath[CHIRP_LINE_MAX];
			sprintf(tempPath ,"%s_row", array->path);
			array->row_oriented = chirp_array_open(array->host, tempPath);
			sprintf(tempPath ,"%s_col", array->path);
			array->col_oriented = chirp_array_open(array->host, tempPath);
			return array;
		}
	}
}

/* return the proprerties of this currently open array */
int chirp_dual_array_width( struct chirp_dual_array *a )
{
	return a->width;
}

int chirp_dual_array_height( struct chirp_dual_array *a )
{
	return a->height;
}

int chirp_dual_array_element_size( struct chirp_dual_array *a )
{
	return a->element_size;
}

int  chirp_dual_array_get( struct chirp_dual_array *a, int i, int j, void *data )
{
	chirp_array_get(a->col_oriented,j,i,data);
	return 0;
}


/* set the indicated element by copying it to the pointer data */
int  chirp_dual_array_set( struct chirp_dual_array *a, int i, int j, const void *data )
{
	chirp_array_set(a->row_oriented,i,j,data);
	chirp_array_set(a->col_oriented,j,i,data);
	return 0;	
}

/* get the indicated row and copy it to the pointer data */
int chirp_dual_array_get_row( struct chirp_dual_array *a, int i, void *data )
{
	return chirp_array_get_row(a->row_oriented,i,data);
}

/* get the indicated col and copy it to the pointer data */
int chirp_dual_array_get_col( struct chirp_dual_array *a, int i, void *data )
{
	return chirp_array_get_col(a->row_oriented,i,data);
}

/* set the indicated row by copying it to the pointer data */
int  chirp_dual_array_set_row( struct chirp_dual_array *a, int i, const void *data )
{
	chirp_array_set_row(a->row_oriented,i,data);
	chirp_array_set_col(a->col_oriented,i,data);
	return 0;
}

/* set the indicated col by copying it to the pointer data */
int chirp_dual_array_set_col( struct chirp_dual_array *a, int i,const void *data )
{
	chirp_array_set_col(a->row_oriented,i,data);
	chirp_array_set_row(a->col_oriented,i,data);
	return 0;
}


int chirp_dual_array_delete( struct chirp_dual_array *a )
{
	int result;
	time_t stoptime = time(0)+300;

	chirp_array_delete(a->row_oriented);
	chirp_array_delete(a->col_oriented);
	char PATH[CHIRP_LINE_MAX];
	sprintf(PATH, "%s%s%s%s",a->path, "/",a->path,".mtdata");
	result = chirp_reli_rmall(a->host,PATH,stoptime);
	result = chirp_reli_rmdir(a->host,a->path,stoptime);

	return 0;
}
